/*
| -----------------------------------------------------------------------------
| Original Filename: serial.c  (from eyebot examples)
| Original Author: Thomas Lampart  (lamparts@autolife.informatik.uni-stuttgart.de)
|
| Modified by: Gerd Troeger  (troeger@informatik.uni-leipzig.de)
|
| Description: 
| serial I/O with C functions
| -----------------------------------------------------------------------------
*/

#include "librobi/librobi.h"
#include "libimpro/improc.h"
#include "librobi/protos.h"

#include "keys.h"
#include "lcd.h"
#include "cam.h"
#include "types.h" 
#include "kern.h"
#include "rs232.h"

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

const BYTE bmp_header[54] = {
	0x42,0x4d,0x46,0x3c,	0x00,0x00,0x00,0x00,	0x00,0x00,0x36,0x00,	0x00,0x00,0x28,0x00,
	0x00,0x00,0x52,0x00,	0x00,0x00,0x3e,0x00,	0x00,0x00,0x01,0x00,	0x18,0x00,0x00,0x00,
	0x00,0x00,0x10,0x3c,	0x00,0x00,0x74,0x12,	0x00,0x00,0x74,0x12,	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,	0x00,0x00 };
const BYTE pad_zero[4] = { 0,0,0,0 };

void mainMenu(void)
{
	LCDClear();
	LCDMode(SCROLLING|NOCURSOR);
	LCDMenu("Ech","Loc","Img","END");
	printf("functions:\n 1) serial echo\n 2) locate red\n 3) send image\n 4) end\n");
}

/*
 * center and filter image
 *
 * Note: it is legal to have src == dest
 */
void filterImage(colimage src, colimage dest)
{
	const BYTE filterMatrix[3][3] = {
		{ 256/16, 256/ 8, 256/16 },
		{ 256/ 8, 256/ 4, 256/ 8 },
		{ 256/16, 256/ 8, 256/16 } };
	int x,y,i;
	colimage tmp;

	/* initally, the image is 80x62, located in the right side...
	   make this 80x60 centered, border=clamp. */
	for (y=1; y<imagerows-1; y++)
	{
		for (i=0; i<3; i++)
			tmp[y][0][i] = src[y][0][i];
		for (x=1; x<imagecolumns-1; x++)
			for (i=0; i<3; i++)
				tmp[y][x][i] = src[y][x-1][i];
		for (i=0; i<3; i++)
			tmp[y][imagecolumns-1][i] = src[y][imagecolumns-1][i];
	}
	y = imagerows-1;
	for (x=0; x<imagecolumns; x++)
		for (i=0; i<3; i++)
		{
			tmp[0][x][i] = tmp[1][x][i];
			tmp[y][x][i] = tmp[y-1][x][i];
		}

	/* blank top/bottem/left/right border of dest */
	for (y=0; y<imagerows; y+=imagerows-1)
		for (x=0; x<imagecolumns-1; x++)
			dest[y][x][0] = dest[y][x][1] = dest[y][x][2] = 0;
	for (y=0; y<imagerows; y++)
		for (x=0; x<imagecolumns; x+=imagecolumns-1)
			dest[y][x][0] = dest[y][x][1] = dest[y][x][2] = 0;

	/* calculate filtered dest image */
	for (y=1; y<imagerows-1; y++)
		for (x=1; x<imagecolumns-1; x++)
			for (i=0; i<3; i++)
			{
				int sum =
					filterMatrix[0][0] * tmp[y-1][x-1][i] +
					filterMatrix[0][1] * tmp[y-1][x  ][i] +
					filterMatrix[0][2] * tmp[y-1][x+1][i] +
					filterMatrix[1][0] * tmp[y  ][x-1][i] +
					filterMatrix[1][1] * tmp[y  ][x  ][i] +
					filterMatrix[1][2] * tmp[y  ][x+1][i] +
					filterMatrix[2][0] * tmp[y+1][x-1][i] +
					filterMatrix[2][1] * tmp[y+1][x  ][i] +
					filterMatrix[2][2] * tmp[y+1][x+1][i] ;
				dest[y][x][i] = sum/256;
			}
}

int locateRedBall(void)
{
	int camera,x,y;
	int sum, sum_x, div, div_x;
	colimage img;

	camera=CAMInit(WIDE);
	if (camera==NOCAM) { printf("ERROR: No camera!\n"); return -1; }
	if (camera==INITERROR) { printf("ERROR: CAMInit!\n"); return -1; }
	if (camera<COLCAM) { printf("ERROR: No Color Camera!\n"); return -1; }

	CAMGetColFrame(&img,FALSE);
	filterImage(img, img);

	sum = 0; div = 0;
	sum_x = 0; div_x = 0;
	for (x=0; x<imagecolumns; x++)
		for (y=0; y<imagerows; y++)
		{
			int t1,t2;
			t1 = img[y][x][0] - img[y][x][1];
			t2 = img[y][x][0] - img[y][x][2];
			if ((t1<=0) || (t2<=0)) continue;	/* this is not "red" */
			sum += x * (t1+t2); div+=(t1+t2);
			sum_x += x; div_x++;
		}

	LCDClear();
	LCDMenu("Loca","te r","ed:","END");
	printf("sum   :%d\n",sum);
	printf("div   :%d\n",div);
	if (div) printf("sum~  :%d\n",sum/div);
	printf("sum_x :%d\n",sum_x);
	printf("div_x :%d\n",div_x);
	if (div_x) printf("sum_x~:%d\n",sum_x/div_x);

/******************/
	return 0;
}

void main ()
{
	FILE    *strm;
	char    buf[20];
	char    z;

/* disable input/output buffer */

	setvbuf (stdout,NULL,_IONBF,0); 
	setvbuf (stdin,NULL,_IONBF,0);  

/* clear display and write message */

	LCDClear();
	LCDMode(SCROLLING|NOCURSOR);
	LCDMenu("","","","END");
	printf("init...\n");

	/* No OSInitRS232, the bios settings are used */

	strm=fopen (SER1,"rw+b");

	if (strm==NULL) printf("Init Error!\n");
	else
	{
		setvbuf (strm,NULL,_IONBF,0);
		mainMenu();
		while (1)
		{
			z=KEYRead();
			if (z==KEY1)
			{
				LCDClear();
				LCDMenu("","","","END");
				printf("Echoing serial:\n");

				while (1)
				{
					memset(buf,0,sizeof(buf));
					if((fscanf(strm,"%s",buf))!=-1) printf("%s\n",buf);
					z=KEYRead();
					if (z==KEY4) break;
				}
				printf("aborting\n");
				OSWait(100);
				mainMenu();
			}
			if (z==KEY2)
			{
				LCDClear();
				LCDMenu("","","","END");
				printf("Locating red:\n");

				while (1)
				{
					locateRedBall();
					z=KEYRead();
					if (z==KEY4) break;
				}
				printf("aborting\n");
				OSWait(100);
				mainMenu();
			}
			if (z==KEY3)
			{
				int camera,x,y,cnt;
				colimage colimg;
				BYTE linebuf[3*imagecolumns+2];

				LCDClear();
				LCDMenu("","","","END");
				printf("Image Transfer:\n");

				camera=CAMInit(WIDE);
				if (camera==NOCAM) printf("ERROR: No camera!\n");
				else if (camera==INITERROR) printf("ERROR: CAMInit!\n");
				else if (camera<COLCAM) printf("ERROR: No Color Camera!\n");
				else {
					CAMGetColFrame(&colimg,FALSE);
					filterImage(colimg, colimg);
					printf("sending image..\n");
					cnt=fwrite(bmp_header,1,54,strm);
					for (y=0; y<imagerows; y++)
					{
						int k=imagerows-y-1;
						for (x=0; x<imagecolumns; x++)
						{
							linebuf[3*x]   = colimg[k][x][2];
							linebuf[3*x+1] = colimg[k][x][1];
							linebuf[3*x+2] = colimg[k][x][0];
						}
						linebuf[3*imagecolumns]   = 0;
						linebuf[3*imagecolumns+1] = 0;
						cnt+=fwrite(linebuf,1,3*imagecolumns+2,strm);
						printf("%d\r",cnt);
					}
					printf("%d:done\n",cnt);
				}
				OSWait(100);
				mainMenu();
			}
			if (z==KEY4) break;
		}
	}       
	printf("Exiting..\n(press any key)");
	getchar();
}
